<!--
SPDX-FileCopyrightText: 2022-2023 nervuri <https://nervuri.net/contact>

SPDX-License-Identifier: BSD-3-Clause
-->

# TLS Client Hello Mirror

This test:

* reflects the complete [Client Hello](https://tls13.xargs.org/#client-hello) message in multiple formats, preserving the order in which TLS parameters and extensions are sent;
* can be used to check for TLS privacy pitfalls ([session resumption](https://svs.informatik.uni-hamburg.de/publications/2018/2018-12-06-Sy-ACSAC-Tracking_Users_across_the_Web_via_TLS_Session_Resumption.pdf), [TLS fingerprinting](https://tlsfingerprint.io/), [system time exposure](https://datatracker.ietf.org/doc/html/draft-mathewson-no-gmtunixtime));
* supports both HTTP and [Gemini](https://geminiprotocol.net/) on the same port;
* is [free as in freedom](https://www.gnu.org/philosophy/free-sw.en.html) and trivial to self-host.

A live instance is running at [tlsprivacy.nervuri.net](https://tlsprivacy.nervuri.net/).

## Installation

See [INSTALL.md](INSTALL.md).

## API documentation

This test exposes two JSON endpoints:

* [/json/v1 (basic)](https://tlsprivacy.nervuri.net/json/v1)
* [/json/v2 (detailed)](https://tlsprivacy.nervuri.net/json/v2)

See [DOC.md](DOC.md) for details.

## Wishlist

* detect client vulnerability to session [prolongation attacks](https://svs.informatik.uni-hamburg.de/publications/2018/2018-12-06-Sy-ACSAC-Tracking_Users_across_the_Web_via_TLS_Session_Resumption.pdf#page=3)
* support early data / 0-RTT (Go's `crypto/tls` library currently does not)
* support sessionID-based resumption (Go's `crypto/tls` library currently does not)
* decode more extensions
* token binding (RFCs 8471-8473, formerly [Channel ID](https://datatracker.ietf.org/doc/html/draft-balfanz-tls-channelid-01)) can be bad for privacy, but Chromium [removed support](https://thenewstack.io/tls-token-binding-standard-gains-a-foothold-on-the-web/) in 2018.  Edge might still support it, though.  It may be worth testing for it (add to highlights and add warning in the UI).

## Contributing

This project is hosted [at tildegit.org](https://tildegit.org/nervuri/client-hello-mirror).  If you don't want to make an account, just [shoot me an email](https://nervuri.net/contact) with your [patch](https://git-send-email.io/) / suggestion / bug report / whatever else.
