// SPDX-FileCopyrightText: 2022-2023 nervuri <https://nervuri.net/contact>
//
// SPDX-License-Identifier: BSD-3-Clause

package clienthello

import "fmt"

// RFC 8879, Section 3
// https://www.iana.org/assignments/tls-extensiontype-values/tls-extensiontype-values.xhtml#tls-certificate-compression-algorithm-ids
var CertComprAlgoList = map[uint16]CertComprAlgoInfo{
	1: {Code: 1, Name: "zlib"},
	2: {Code: 2, Name: "brotli"},
	3: {Code: 3, Name: "zstd"},
}

type CertComprAlgo any // (uint16 | CertComprAlgoInfo)

type CertComprAlgoInfo struct {
	Code uint16 `json:"code"`
	Name string `json:"name"`
}

func GetCertComprAlgoInfo(algoCode uint16, mustName bool) CertComprAlgoInfo {
	info, found := CertComprAlgoList[algoCode]
	if !found {
		info = CertComprAlgoInfo{
			Code: algoCode,
		}
	}
	if mustName && info.Name == "" {
		info.Name = fmt.Sprint("(", info.Code, ")")
	}
	return info
}

func (m *ClientHelloMsg) AddCertComprAlgoInfo() {
	for i, ext := range m.Extensions {
		if ext.Code == extensionCompressCertificate {
			for j, algoCode := range ext.Data.CertificateCompressionAlgos {
				m.Extensions[i].Data.CertificateCompressionAlgos[j] =
					GetCertComprAlgoInfo(algoCode.(uint16), false)
			}
		}
	}
}

func (m *ClientHelloMsg) GetCertComprAlgos() []CertComprAlgo {
	var algos []CertComprAlgo
	for _, ext := range m.Extensions {
		if ext.Code == extensionCompressCertificate {
			algos = append(algos, ext.Data.CertificateCompressionAlgos...)
		}
	}
	return algos
}
