// SPDX-FileCopyrightText: 2022-2023 nervuri <https://nervuri.net/contact>
//
// SPDX-License-Identifier: BSD-3-Clause

// PSK Key Exchange Modes

package clienthello

// https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-pskkeyexchangemode
// https://www.rfc-editor.org/rfc/rfc8446.html#section-4.2.9
var PSKModeList = map[uint8]PSKModeInfo{
	0: {Code: 0, Name: "psk_ke", Recommended: true, Reference: "[RFC8446]"},
	1: {Code: 1, Name: "psk_dhe_ke", Recommended: true, Reference: "[RFC8446]"},

	0x0B: {Code: 0x0B, Name: "GREASE", Recommended: false, Reference: "[RFC8701]"},
	0x2A: {Code: 0x2A, Name: "GREASE", Recommended: false, Reference: "[RFC8701]"},
	0x49: {Code: 0x49, Name: "GREASE", Recommended: false, Reference: "[RFC8701]"},
	0x68: {Code: 0x68, Name: "GREASE", Recommended: false, Reference: "[RFC8701]"},
	0x87: {Code: 0x87, Name: "GREASE", Recommended: false, Reference: "[RFC8701]"},
	0xA6: {Code: 0xA6, Name: "GREASE", Recommended: false, Reference: "[RFC8701]"},
	0xC5: {Code: 0xC5, Name: "GREASE", Recommended: false, Reference: "[RFC8701]"},
	0xE4: {Code: 0xE4, Name: "GREASE", Recommended: false, Reference: "[RFC8701]"},
}

type PSKMode any // (uint8 | PSKModeInfo)

type PSKModeInfo struct {
	Code        uint8  `json:"code"`
	Name        string `json:"name"`
	Recommended bool   `json:"-"`
	Reference   string `json:"-"`
}

func GetPSKModeInfo(pskModeCode uint8) PSKModeInfo {
	info, found := PSKModeList[pskModeCode]
	if !found {
		info = PSKModeInfo{
			Code: pskModeCode,
		}
	}
	return info
}

func (m *ClientHelloMsg) AddPSKModeInfo() {
	for i, ext := range m.Extensions {
		if ext.Code == extensionPSKModes {
			for j, pskMode := range ext.Data.PSKModes {
				m.Extensions[i].Data.PSKModes[j] = GetPSKModeInfo(pskMode.(uint8))
			}
		}
	}
}
